package fms

import (
	"hongze/fms_api/global"
	"hongze/fms_api/utils"
	"time"
)

// ContractRegisterLog 合同登记操作记录表
type ContractRegisterLog struct {
	Id                 int       `gorm:"primaryKey;column:id" json:"id" description:"记录ID"`
	ContractRegisterId int       `gorm:"column:contract_register_id" json:"contract_register_id" description:"登记ID"`
	AdminId            int       `gorm:"column:admin_id" json:"admin_id" description:"操作人ID"`
	AdminName          string    `gorm:"column:admin_name" json:"admin_name" description:"当时操作人姓名"`
	OpData             string    `gorm:"column:op_data" json:"op_data" description:"操作数据-JSON"`
	OpType             int       `gorm:"column:op_type" json:"op_type" description:"操作类型: 1-合规登记; 2-开票登记; 3-到款登记; 4-修改合同状态; 5-删除合同登记; 6-合规编辑"`
	Remark             string    `gorm:"column:remark" json:"remark" description:"备注信息"`
	AmountRemark       string    `gorm:"column:amount_remark" json:"amount_remark" description:"金额描述"`
	CreateTime         time.Time `gorm:"autoCreateTime;column:create_time" json:"create_time" description:"创建时间"`
}

// ContractRegisterLogItem 合同登记-操作记录表
type ContractRegisterLogItem struct {
	ContractRegisterId int    `gorm:"column:contract_register_id" json:"contract_register_id" description:"登记ID"`
	AdminId            int    `gorm:"column:admin_id" json:"admin_id" description:"操作人ID"`
	AdminName          string `gorm:"column:admin_name" json:"admin_name" description:"当时操作人姓名"`
	OpData             string `gorm:"column:op_data" json:"-" description:"操作数据-JSON"`
	OpType             int    `gorm:"column:op_type" json:"op_type" description:"操作类型: 1-合规登记; 2-开票登记; 3-到款登记; 4-修改合同状态; 5-删除合同登记;"`
	Remark             string `gorm:"column:remark" json:"remark" description:"备注信息"`
	AmountRemark       string `gorm:"column:amount_remark" json:"amount_remark" description:"金额描述"`
	CreateTime         string `gorm:"autoCreateTime;column:create_time" json:"create_time" description:"创建时间"`
}

func (c *ContractRegisterLog) TableName() string {
	return "contract_register_log"
}

func (c *ContractRegisterLog) Create() (err error) {
	err = global.DEFAULT_MYSQL.Create(c).Error
	return
}

func (c *ContractRegisterLog) AddInBatches(list []*ContractRegisterLog) (err error) {
	err = global.DEFAULT_MYSQL.CreateInBatches(list, len(list)).Error
	return
}

func (c *ContractRegisterLog) List(condition string, pars []interface{}) (list []*ContractRegisterLog, err error) {
	list = make([]*ContractRegisterLog, 0)
	err = global.DEFAULT_MYSQL.Model(c).
		Where(condition, pars...).
		Find(&list).Error
	return
}

// GetContractRegisterLogItemList 获取合同登记操作记录
func GetContractRegisterLogItemList(condition string, pars []interface{}) (itemList []*ContractRegisterLogItem, err error) {
	list := make([]*ContractRegisterLog, 0)
	err = global.DEFAULT_MYSQL.Model(ContractRegisterLog{}).
		Where(condition, pars...).
		Order("create_time DESC").
		Find(&list).Error
	if err != nil {
		return
	}
	itemList = formatContractRegisterLog2ItemList(list)
	return
}

// formatContractRegisterLog2ItemList 格式化ContractRegisterLog
func formatContractRegisterLog2ItemList(list []*ContractRegisterLog) (itemList []*ContractRegisterLogItem) {
	itemList = make([]*ContractRegisterLogItem, 0)
	for i := range list {
		itemList = append(itemList, &ContractRegisterLogItem{
			ContractRegisterId: list[i].ContractRegisterId,
			AdminId:            list[i].AdminId,
			AdminName:          list[i].AdminName,
			OpData:             list[i].OpData,
			OpType:             list[i].OpType,
			Remark:             list[i].Remark,
			AmountRemark:       list[i].AmountRemark,
			CreateTime:         utils.TimeTransferString(utils.FormatDateTime, list[i].CreateTime),
		})
	}
	return
}
